@@ -11,7 +11,7 @@
 # This file implements some common TCL routines used for regression
 # testing the SQLite library
 #
-# $Id: tester.tcl,v 1.141 2009/03/26 17:13:06 danielk1977 Exp $
+# $Id: tester.tcl,v 1.143 2009/04/09 01:23:49 drh Exp $
 
 #
 # What for user input before continuing.  This gives an opportunity
@@ -132,6 +132,7 @@ if {![info exists nTest]} {
   sqlite3_shutdown 
   install_malloc_faultsim 1 
   sqlite3_initialize
+  autoinstall_test_functions
   if {[info exists tester_do_binarylog]} {
     sqlite3_instvfs binarylog -default binarylog ostrace.bin
     sqlite3_instvfs marker binarylog "$argv0 $argv"
@@ -644,7 +645,7 @@ proc do_ioerr_test {testname args} {
   set ::go 1
   #reset_prng_state
   save_prng_state
-  for {set n $::ioerropts(-start)} {$::go && $n<200} {incr n} {
+  for {set n $::ioerropts(-start)} {$::go} {incr n} {
     set ::TN $n
     incr ::ioerropts(-count) -1
     if {$::ioerropts(-count)<0} break
@@ -957,6 +958,20 @@ proc copy_file {from to} {
   }
 }
 
+# Drop all tables in database [db]
+proc drop_all_tables {{db db}} {
+  set pk [$db one "PRAGMA foreign_keys"]
+  $db eval "PRAGMA foreign_keys = OFF"
+  foreach {t type} [$db eval {
+    SELECT name, type FROM sqlite_master 
+    WHERE type IN('table', 'view') AND name NOT like 'sqlite_%'
+  }] {
+    $db eval "DROP $type $t"
+  }
+  $db eval " PRAGMA foreign_keys = $pk "
+}
+
+
 # If the library is compiled with the SQLITE_DEFAULT_AUTOVACUUM macro set
 # to non-zero, then set the global variable $AUTOVACUUM to 1.
 set AUTOVACUUM $sqlite_options(default_autovacuum)
